#ifndef CREATURELIB_TYPELIBRARY_HPP
#define CREATURELIB_TYPELIBRARY_HPP

#include <Arbutils/Collections/Dictionary.hpp>
#include <Arbutils/Collections/List.hpp>
#include <Arbutils/String/StringView.hpp>
#include <numeric>
#include "../Defines.hpp"
#include "Exceptions/CreatureException.hpp"

namespace CreatureLib::Library {
    class TypeLibrary {
        ArbUt::Dictionary<ArbUt::StringView, u8> _types;
        ArbUt::List<ArbUt::List<float>> _effectiveness;

    public:
        TypeLibrary(size_t initialCapacity = 20) : _types(ArbUt::Dictionary<ArbUt::StringView, u8>(initialCapacity)) {}

        inline uint8_t GetTypeId(const ArbUt::StringView& key) const { return _types.Get(key); }
        [[nodiscard]] inline float GetSingleEffectiveness(u8 attacking, u8 defensive) const {
            try {
                return _effectiveness[attacking][defensive];
            } catch (const std::exception& e) {
                THROW("Unknown type indices were requested for effectiveness: ", (uint32_t)attacking, " and ",
                      (uint32_t)defensive);
            }
        }
        [[nodiscard]] inline float GetEffectiveness(uint8_t attacking, const std::vector<u8>& defensive) const {
            return std::accumulate(defensive.begin(), defensive.end(), (float)1,
                                   [this, attacking](float init, uint8_t defense) {
                                       return init * GetSingleEffectiveness(attacking, defense);
                                   });
        }
        const ArbUt::StringView& GetTypeName(u8 type) const;

        uint8_t RegisterType(const ArbUt::StringView& typeName);
        void SetEffectiveness(uint8_t attacking, u8 defensive, float effectiveness);
    };
}

#endif // CREATURELIB_TYPELIBRARY_HPP
